@@ -1,6 +1,7 @@
 /*
  *
  * Copyright (C) 2003-2009 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 2010 Michal Malek <michalm@jabster.pl>
  *
  * This file is part of the K3b project.
  * Copyright (C) 1998-2009 Sebastian Trueg <trueg@k3b.org>
@@ -39,7 +40,7 @@ namespace {
     {
         return bin1->version > bin2->version;
     }
-    
+
     const int EXECUTE_TIMEOUT = 5000; // in seconds
 }
 
@@ -110,8 +111,7 @@ K3b::ExternalProgram* K3b::ExternalBin::program() const
 
 
 K3b::ExternalProgram::ExternalProgram( const QString& name )
-    : m_name( name ),
-      m_defaultBin( 0 )
+    : m_name( name )
 {
 }
 
@@ -135,7 +135,17 @@ const K3b::ExternalBin* K3b::ExternalProgram::mostRecentBin() const
 
 const K3b::ExternalBin* K3b::ExternalProgram::defaultBin() const
 {
-    return m_defaultBin;
+    if( m_bins.size() == 1 ) {
+        return m_bins.first();
+    }
+    else {
+        for( QList<const K3b::ExternalBin*>::const_iterator it = m_bins.constBegin(); it != m_bins.constEnd(); ++it ) {
+            if( ( *it )->path == m_defaultBin ) {
+                return *it;
+            }
+        }
+        return 0;
+    }
 }
 
 
@@ -148,8 +158,9 @@ void K3b::ExternalProgram::addBin( K3b::ExternalBin* bin )
         // so we default to using the newest one
         qSort( m_bins.begin(), m_bins.end(), compareVersions );
 
-        if ( !m_defaultBin || bin->version > m_defaultBin->version ) {
-            m_defaultBin = bin;
+        const ExternalBin* defBin = defaultBin();
+        if ( !defBin || bin->version > defBin->version ) {
+            setDefault( bin );
         }
     }
 }
@@ -157,20 +168,18 @@ void K3b::ExternalProgram::addBin( K3b::ExternalBin* bin )
 
 void K3b::ExternalProgram::setDefault( const K3b::ExternalBin* bin )
 {
-    if ( bin ) {
-        m_defaultBin = bin;
+    for( QList<const K3b::ExternalBin*>::const_iterator it = m_bins.constBegin(); it != m_bins.constEnd(); ++it ) {
+        if( *it == bin ) {
+            m_defaultBin = (*it)->path;
+            break;
+        }
     }
 }
 
 
 void K3b::ExternalProgram::setDefault( const QString& path )
 {
-    for( QList<const K3b::ExternalBin*>::const_iterator it = m_bins.constBegin(); it != m_bins.constEnd(); ++it ) {
-        if( ( *it )->path == path ) {
-            setDefault( *it );
-            return;
-        }
-    }
+    m_defaultBin = path;
 }
 
 
@@ -201,7 +210,6 @@ QString K3b::ExternalProgram::buildProgramPath( const QString& dir, const QStrin
 class K3b::SimpleExternalProgram::Private
 {
 public:
-    QString versionIdentifier;
 };
 
 
@@ -218,7 +226,7 @@ K3b::SimpleExternalProgram::~SimpleExternalProgram()
 }
 
 
-QString K3b::SimpleExternalProgram::getProgramPath( const QString& dir )
+QString K3b::SimpleExternalProgram::getProgramPath( const QString& dir ) const
 {
     return buildProgramPath( dir, name() );
 }
@@ -235,13 +243,13 @@ bool K3b::SimpleExternalProgram::scan( const QString& p )
         K3b::ExternalBin* bin = new ExternalBin( this );
         bin->path = path;
 
-        if ( !scanVersion( bin ) ||
-             !scanFeatures( bin ) ) {
+        if ( !scanVersion( *bin ) ||
+             !scanFeatures( *bin ) ) {
             delete bin;
             return false;
         }
 
-        addBin(bin);
+        addBin( bin );
         return true;
     }
     else {
@@ -250,37 +258,37 @@ bool K3b::SimpleExternalProgram::scan( const QString& p )
 }
 
 
-bool K3b::SimpleExternalProgram::scanVersion( ExternalBin* bin )
+bool K3b::SimpleExternalProgram::scanVersion( ExternalBin& bin ) const
 {
     // probe version
     KProcess vp;
     vp.setOutputChannelMode( KProcess::MergedChannels );
-    vp << bin->path << "--version";
+    vp << bin.path << "--version";
     if( vp.execute( EXECUTE_TIMEOUT ) < 0 )
         return false;
 
     QString s = QString::fromLocal8Bit( vp.readAll() );
-    bin->version = parseVersion( s );
-    bin->copyright = parseCopyright( s );
-    return bin->version.isValid();
+    bin.version = parseVersion( s, bin );
+    bin.copyright = parseCopyright( s, bin );
+    return bin.version.isValid();
 }
 
 
-bool K3b::SimpleExternalProgram::scanFeatures( ExternalBin* bin )
+bool K3b::SimpleExternalProgram::scanFeatures( ExternalBin& bin ) const
 {
 #ifndef Q_OS_WIN32
     // check if we run as root
     struct stat s;
-    if( !::stat( QFile::encodeName(bin->path), &s ) ) {
+    if( !::stat( QFile::encodeName(bin.path), &s ) ) {
         if( (s.st_mode & S_ISUID) && s.st_uid == 0 )
-            bin->addFeature( "suidroot" );
+            bin.addFeature( "suidroot" );
     }
 #endif
 
     // probe features
     KProcess fp;
     fp.setOutputChannelMode( KProcess::MergedChannels );
-    fp << bin->path << "--help";
+    fp << bin.path << "--help";
     if( fp.execute( EXECUTE_TIMEOUT ) < 0 )
         return false;
 
@@ -289,11 +297,11 @@ bool K3b::SimpleExternalProgram::scanFeatures( ExternalBin* bin )
 }
 
 
-K3b::Version K3b::SimpleExternalProgram::parseVersion( const QString& out )
+K3b::Version K3b::SimpleExternalProgram::parseVersion( const QString& out, const ExternalBin& bin ) const
 {
     // we first look for the program name with first upper char so we do not catch
     // the warning messages on stderr (cdrecord sometimes produces those)
-    QString programName = d->versionIdentifier.isEmpty() ? name() : d->versionIdentifier;
+    QString programName = versionIdentifier( bin );
     QString programNameCap = programName[0].toUpper() + programName.mid( 1 );
     int pos = out.indexOf( programNameCap );
     if ( pos < 0 )
@@ -306,29 +314,23 @@ K3b::Version K3b::SimpleExternalProgram::parseVersion( const QString& out )
 }
 
 
-QString K3b::SimpleExternalProgram::parseCopyright( const QString& out )
+QString K3b::SimpleExternalProgram::parseCopyright( const QString& output, const ExternalBin& /*bin*/ ) const
 {
-    int pos = out.indexOf( "(C)", 0 );
+    int pos = output.indexOf( "(C)", 0 );
     if ( pos < 0 )
         return QString();
     pos += 4;
-    int endPos = out.indexOf( '\n', pos );
-    return out.mid( pos, endPos-pos );
+    int endPos = output.indexOf( '\n', pos );
+    return output.mid( pos, endPos-pos );
 }
 
 
-void K3b::SimpleExternalProgram::parseFeatures( const QString&, ExternalBin* )
+void K3b::SimpleExternalProgram::parseFeatures( const QString& /*output*/, ExternalBin& /*bin*/ ) const
 {
     // do nothing
 }
 
 
-void K3b::SimpleExternalProgram::setVersionIdentifier( const QString& s )
-{
-    d->versionIdentifier = s;
-}
-
-
 // static
 K3b::Version K3b::SimpleExternalProgram::parseVersionAt( const QString& data, int pos )
 {
@@ -344,6 +346,12 @@ K3b::Version K3b::SimpleExternalProgram::parseVersionAt( const QString& data, in
 }
 
 
+QString K3b::SimpleExternalProgram::versionIdentifier( const ExternalBin& /*bin*/ ) const
+{
+    return name();
+}
+
+
 // ///////////////////////////////////////////////////////////
 //
 // K3BEXTERNALBINMANAGER
@@ -496,7 +504,7 @@ K3b::ExternalProgram* K3b::ExternalBinManager::program( const QString& name ) co
 
 void K3b::ExternalBinManager::loadDefaultSearchPath()
 {
-    static const char* defaultSearchPaths[] = {
+    static const char* const defaultSearchPaths[] = {
 #ifndef Q_OS_WIN32
                                                 "/usr/bin/",
                                                 "/usr/local/bin/",
@@ -516,12 +524,9 @@ void K3b::ExternalBinManager::loadDefaultSearchPath()
 
 void K3b::ExternalBinManager::setSearchPath( const QStringList& list )
 {
-    loadDefaultSearchPath();
-
+    m_searchPath.clear();
     for( QStringList::const_iterator it = list.constBegin(); it != list.constEnd(); ++it ) {
-        QString aPath = QDir::fromNativeSeparators( *it );
-        if( !m_searchPath.contains( aPath ) )
-            m_searchPath.append( aPath );
+        m_searchPath.append( QDir::fromNativeSeparators( *it ) );
     }
 }
 
