@@ -58,7 +58,7 @@ class K3b::DataJob::Private
 {
 public:
     Private()
-        : usedWritingApp(K3b::WritingAppCdrecord),
+        : usedWritingApp(K3b::WritingAppAuto),
           verificationJob( 0 ),
           pipe( 0 ) {
     }
@@ -716,7 +716,7 @@ bool K3b::DataJob::waitForBurnMedium()
                                                    K3b::Device::STATE_INCOMPLETE :
                                                    K3b::Device::STATE_EMPTY,
                                                    m,
-                                                   d->doc->length() );
+                                                   d->doc->burningLength() );
 
     if( foundMedium == Device::MEDIA_UNKNOWN || hasBeenCanceled() ) {
         return false;
@@ -813,8 +813,12 @@ bool K3b::DataJob::waitForBurnMedium()
 
         d->usedWritingApp = writingApp();
         // let's default to cdrecord for the time being (except for special cases below)
+        // but prefer growisofs to wodim for DVDs
         if ( d->usedWritingApp == K3b::WritingAppAuto ) {
-            d->usedWritingApp = K3b::WritingAppCdrecord;
+            if (k3bcore->externalBinManager()->binObject("cdrecord")->hasFeature( "wodim" ))
+                d->usedWritingApp = K3b::WritingAppGrowisofs;
+            else
+                d->usedWritingApp = K3b::WritingAppCdrecord;
         }
 
         // -------------------------------
@@ -843,8 +847,8 @@ bool K3b::DataJob::waitForBurnMedium()
             }
 
             if( foundMedium & K3b::Device::MEDIA_DVD_PLUS_RW &&
-                ( usedMultiSessionMode() == K3b::DataDoc::NONE ||
-                  usedMultiSessionMode() == K3b::DataDoc::START ) )
+                ( usedMultiSessionMode() == K3b::DataDoc::CONTINUE ||
+                  usedMultiSessionMode() == K3b::DataDoc::FINISH ) )
                 emit infoMessage( i18n("Growing ISO9660 filesystem on %1.", Device::mediaTypeString(foundMedium, true)), MessageInfo );
             else
                 emit infoMessage( i18n("Writing %1.", Device::mediaTypeString(foundMedium, true)), MessageInfo );
@@ -930,7 +934,10 @@ bool K3b::DataJob::waitForBurnMedium()
     else if ( foundMedium & K3b::Device::MEDIA_BD_ALL ) {
         d->usedWritingApp = writingApp();
         if( d->usedWritingApp == K3b::WritingAppAuto ) {
-            d->usedWritingApp = K3b::WritingAppCdrecord;
+            if (k3bcore->externalBinManager()->binObject("cdrecord")->hasFeature( "wodim" ))
+                d->usedWritingApp = K3b::WritingAppGrowisofs;
+            else
+                d->usedWritingApp = K3b::WritingAppCdrecord;
         }
 
         if ( d->usedWritingApp == K3b::WritingAppCdrecord &&
@@ -961,8 +968,8 @@ bool K3b::DataJob::waitForBurnMedium()
         }
 
         if( foundMedium & K3b::Device::MEDIA_BD_RE &&
-            ( usedMultiSessionMode() == K3b::DataDoc::NONE ||
-                usedMultiSessionMode() == K3b::DataDoc::START ) )
+            ( usedMultiSessionMode() == K3b::DataDoc::CONTINUE ||
+              usedMultiSessionMode() == K3b::DataDoc::FINISH ) )
             emit infoMessage( i18n("Growing ISO9660 filesystem on %1.", Device::mediaTypeString(foundMedium, true)), MessageInfo );
         else
             emit infoMessage( i18n("Writing %1.", Device::mediaTypeString(foundMedium, true)), MessageInfo );
@@ -1153,7 +1160,7 @@ bool K3b::DataJob::setupGrowisofsJob()
     if( usedMultiSessionMode() != K3b::DataDoc::NONE ) {
         //
         // growisofs wants a valid -C parameter for multisession, so we get it from the
-        // K3b::MsInfoFetcher (see K3b::DataJob::slotMsInfoFetched)
+        // K3b::MsInfoFetcher (see K3b::DataJob::prepareWriting)
         //
         writer->setMultiSessionInfo( m_isoImager->multiSessionInfo() );
     }
