@@ -17,6 +17,13 @@
 #include <config-k3b.h>
 
 extern "C" {
+/*
+ Recent versions of FFmepg uses C99 constant macros which are not presebt in C++ standard.
+ The macro __STDC_CONSTANT_MACROS allow C++ to use these macros. Altough it's not defined by C++ standard
+ it's supported by many implementations.
+ See bug 236036 and discussion: http://lists.mplayerhq.hu/pipermail/ffmpeg-devel/2010-May/088074.html
+ */
+#define __STDC_CONSTANT_MACROS
 #ifdef NEWFFMPEGAVCODECPATH
 #include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
@@ -32,9 +39,40 @@ extern "C" {
 
 
 #if LIBAVFORMAT_BUILD < 4629
-#define FFMPEG_BUILD_PRE_4629
+#define FFMPEG_CODEC(s) (&s->codec)
+#else
+#define FFMPEG_CODEC(s) (s->codec)
 #endif
 
+#ifndef HAVE_FFMPEG_AVFORMAT_OPEN_INPUT
+//      this works because the parameters/options are not used
+#  define avformat_open_input(c,s,f,o) av_open_input_file(c,s,f,0,o)
+#endif
+#ifndef HAVE_FFMPEG_AV_DUMP_FORMAT
+#  define av_dump_format(c,x,f,y) dump_format(c,x,f,y)
+#endif
+#ifndef HAVE_FFMPEG_AVFORMAT_FIND_STREAM_INFO
+#  define avformat_find_stream_info(c,o) av_find_stream_info(c)
+#endif
+#ifndef HAVE_FFMPEG_AVFORMAT_CLOSE_INPUT
+#  define avformat_close_input(c) av_close_input_file(*c)
+#endif
+#ifndef HAVE_FFMPEG_AVCODEC_OPEN2
+#  define avcodec_open2(a,c,o) avcodec_open(a,c)
+#endif
+#ifndef HAVE_FFMPEG_AVMEDIA_TYPE
+#  define AVMEDIA_TYPE_AUDIO CODEC_TYPE_AUDIO
+#endif
+#ifndef HAVE_FFMPEG_CODEC_MP3
+#  define CODEC_ID_MP3 CODEC_ID_MP3LAME
+#endif
+
+// Has been deprecated for ages. Now removed. For now do what ffmpeg does
+// http://ffmpeg.org/pipermail/ffmpeg-cvslog/2012-August/053785.html
+// 192000 = 1 second of 48khz 32bit audio
+#ifndef AVCODEC_MAX_AUDIO_FRAME_SIZE
+#define AVCODEC_MAX_AUDIO_FRAME_SIZE 192000
+#endif
 
 K3bFFMpegWrapper* K3bFFMpegWrapper::s_instance = 0;
 
@@ -42,16 +80,17 @@ K3bFFMpegWrapper* K3bFFMpegWrapper::s_instance = 0;
 class K3bFFMpegFile::Private
 {
 public:
-    AVFormatContext* formatContext;
-    AVCodec* codec;
+    ::AVFormatContext* formatContext;
+    ::AVCodec* codec;
 
     K3b::Msf length;
 
-    // for decoding
-    char outputBuffer[AVCODEC_MAX_AUDIO_FRAME_SIZE];
+    // for decoding. ffmpeg requires 16-byte alignment.
+    char outputBuffer[AVCODEC_MAX_AUDIO_FRAME_SIZE + 15];
+    char* alignedOutputBuffer;
     char* outputBufferPos;
     int outputBufferSize;
-    AVPacket packet;
+    ::AVPacket packet;
     quint8* packetData;
     int packetSize;
 };
@@ -63,6 +102,8 @@ K3bFFMpegFile::K3bFFMpegFile( const QString& filename )
     d = new Private;
     d->formatContext = 0;
     d->codec = 0;
+    int offset = 0x10 - (reinterpret_cast<intptr_t>(&d->outputBuffer) & 0xf);
+    d->alignedOutputBuffer = &d->outputBuffer[offset];
 }
 
 
@@ -78,14 +119,14 @@ bool K3bFFMpegFile::open()
     close();
 
     // open the file
-    int err = av_open_input_file( &d->formatContext, m_filename.toLocal8Bit(), 0, 0, 0 );
+    int err = ::avformat_open_input( &d->formatContext, m_filename.toLocal8Bit(), 0, 0 );
     if( err < 0 ) {
         kDebug() << "(K3bFFMpegFile) unable to open " << m_filename << " with error " << err;
         return false;
     }
 
     // analyze the streams
-    av_find_stream_info( d->formatContext );
+    ::avformat_find_stream_info( d->formatContext, 0 );
 
     // we only handle files containing one audio stream
     if( d->formatContext->nb_streams != 1 ) {
@@ -94,18 +135,15 @@ bool K3bFFMpegFile::open()
     }
 
     // urgh... ugly
-#ifdef FFMPEG_BUILD_PRE_4629
-    AVCodecContext* codecContext =  &d->formatContext->streams[0]->codec;
-#else
-    AVCodecContext* codecContext =  d->formatContext->streams[0]->codec;
-#endif
-    if( codecContext->codec_type != CODEC_TYPE_AUDIO ) {
+    ::AVCodecContext* codecContext =  FFMPEG_CODEC(d->formatContext->streams[0]);
+    if( codecContext->codec_type != AVMEDIA_TYPE_AUDIO)
+    {
         kDebug() << "(K3bFFMpegFile) not a simple audio stream: " << m_filename;
         return false;
     }
 
     // get the codec
-    d->codec = avcodec_find_decoder(codecContext->codec_id);
+    d->codec = ::avcodec_find_decoder(codecContext->codec_id);
     if( !d->codec ) {
         kDebug() << "(K3bFFMpegFile) no codec found for " << m_filename;
         return false;
@@ -113,7 +151,7 @@ bool K3bFFMpegFile::open()
 
     // open the codec on our context
     kDebug() << "(K3bFFMpegFile) found codec for " << m_filename;
-    if( avcodec_open( codecContext, d->codec ) < 0 ) {
+    if( ::avcodec_open2( codecContext, d->codec, 0 ) < 0 ) {
         kDebug() << "(K3bFFMpegDecoderFactory) could not open codec.";
         return false;
     }
@@ -127,7 +165,7 @@ bool K3bFFMpegFile::open()
     }
 
     // dump some debugging info
-    dump_format( d->formatContext, 0, m_filename.toLocal8Bit(), 0 );
+    ::av_dump_format( d->formatContext, 0, m_filename.toLocal8Bit(), 0 );
 
     return true;
 }
@@ -140,16 +178,12 @@ void K3bFFMpegFile::close()
     d->packetData = 0;
 
     if( d->codec ) {
-#ifdef FFMPEG_BUILD_PRE_4629
-        avcodec_close( &d->formatContext->streams[0]->codec );
-#else
-        avcodec_close( d->formatContext->streams[0]->codec );
-#endif
+        ::avcodec_close( FFMPEG_CODEC(d->formatContext->streams[0]) );
         d->codec = 0;
     }
 
     if( d->formatContext ) {
-        av_close_input_file( d->formatContext );
+        ::avformat_close_input( &d->formatContext );
         d->formatContext = 0;
     }
 }
@@ -163,48 +197,38 @@ K3b::Msf K3bFFMpegFile::length() const
 
 int K3bFFMpegFile::sampleRate() const
 {
-#ifdef FFMPEG_BUILD_PRE_4629
-    return d->formatContext->streams[0]->codec.sample_rate;
-#else
-    return d->formatContext->streams[0]->codec->sample_rate;
-#endif
+    return FFMPEG_CODEC(d->formatContext->streams[0])->sample_rate;
 }
 
 
 int K3bFFMpegFile::channels() const
 {
-#ifdef FFMPEG_BUILD_PRE_4629
-    return d->formatContext->streams[0]->codec.channels;
-#else
-    return d->formatContext->streams[0]->codec->channels;
-#endif
+    return FFMPEG_CODEC(d->formatContext->streams[0])->channels;
 }
 
 
 int K3bFFMpegFile::type() const
 {
-#ifdef FFMPEG_BUILD_PRE_4629
-    return d->formatContext->streams[0]->codec.codec_id;
-#else
-    return d->formatContext->streams[0]->codec->codec_id;
-#endif
+    return FFMPEG_CODEC(d->formatContext->streams[0])->codec_id;
 }
 
 
 QString K3bFFMpegFile::typeComment() const
 {
-    switch( type() ) {
-    case CODEC_ID_WMAV1:
-        return i18n("Windows Media v1");
-    case CODEC_ID_WMAV2:
-        return i18n("Windows Media v2");
-#if LIBAVCODEC_VERSION_MAJOR < 52
-    case CODEC_ID_MP3LAME:
-#else
-    case CODEC_ID_MP3:
+#if LIBAVCODEC_BUILD < AV_VERSION_INT(54,25,0)
+    #define AV_CODEC_ID_WMAV1  CODEC_ID_WMAV1
+    #define AV_CODEC_ID_WMAV2  CODEC_ID_WMAV2
+    #define AV_CODEC_ID_MP3    CODEC_ID_MP3
+    #define AV_CODEC_ID_AAC    CODEC_ID_AAC
 #endif
+    switch( type() ) {
+    case AV_CODEC_ID_WMAV1:
+        return i18n("Windows Media v1");
+    case AV_CODEC_ID_WMAV2:
+        return i18n("Windows Media v2");
+    case AV_CODEC_ID_MP3:
         return i18n("MPEG 1 Layer III");
-    case CODEC_ID_AAC:
+    case AV_CODEC_ID_AAC:
         return i18n("Advanced Audio Coding (AAC)");
     default:
         return QString::fromLocal8Bit( d->codec->name );
@@ -215,8 +239,11 @@ QString K3bFFMpegFile::typeComment() const
 QString K3bFFMpegFile::title() const
 {
     // FIXME: is this UTF8 or something??
-    if( d->formatContext->title[0] != '\0' )
-        return QString::fromLocal8Bit( d->formatContext->title );
+    AVDictionaryEntry *ade = av_dict_get( d->formatContext->metadata, "TITLE", NULL, 0 );
+    if( ade == NULL )
+        return QString();
+    if( ade->value != '\0' )
+        return QString::fromLocal8Bit( ade->value );
     else
         return QString();
 }
@@ -225,8 +252,11 @@ QString K3bFFMpegFile::title() const
 QString K3bFFMpegFile::author() const
 {
     // FIXME: is this UTF8 or something??
-    if( d->formatContext->author[0] != '\0' )
-        return QString::fromLocal8Bit( d->formatContext->author );
+    AVDictionaryEntry *ade = av_dict_get( d->formatContext->metadata, "ARTIST", NULL, 0 );
+    if( ade == NULL )
+        return QString();
+    if( ade->value != '\0' )
+        return QString::fromLocal8Bit( ade->value );
     else
         return QString();
 }
@@ -235,8 +265,11 @@ QString K3bFFMpegFile::author() const
 QString K3bFFMpegFile::comment() const
 {
     // FIXME: is this UTF8 or something??
-    if( d->formatContext->comment[0] != '\0' )
-        return QString::fromLocal8Bit( d->formatContext->comment );
+    AVDictionaryEntry *ade = av_dict_get( d->formatContext->metadata, "COMMENT", NULL, 0 );
+    if( ade == NULL )
+        return QString();
+    if( ade->value != '\0' )
+        return QString::fromLocal8Bit( ade->value );
     else
         return QString();
 }
@@ -244,23 +277,24 @@ QString K3bFFMpegFile::comment() const
 
 int K3bFFMpegFile::read( char* buf, int bufLen )
 {
-    if( fillOutputBuffer() > 0 ) {
-        int len = qMin(bufLen, d->outputBufferSize);
-        ::memcpy( buf, d->outputBufferPos, len );
-
-        // TODO: only swap if needed
-        for( int i = 0; i < len-1; i+=2 ) {
-            char a = buf[i];
-            buf[i] = buf[i+1];
-            buf[i+1] = a;
-        }
-
-        d->outputBufferPos += len;
-        d->outputBufferSize -= len;
-        return len;
+    int ret = fillOutputBuffer();
+    if (ret <= 0) {
+        return ret;
     }
-    else
-        return 0;
+
+    int len = qMin(bufLen, d->outputBufferSize);
+    ::memcpy( buf, d->outputBufferPos, len );
+
+    // TODO: only swap if needed
+    for( int i = 0; i < len-1; i+=2 ) {
+        char a = buf[i];
+        buf[i] = buf[i+1];
+        buf[i+1] = a;
+    }
+
+    d->outputBufferPos += len;
+    d->outputBufferSize -= len;
+    return len;
 }
 
 
@@ -268,12 +302,11 @@ int K3bFFMpegFile::read( char* buf, int bufLen )
 int K3bFFMpegFile::readPacket()
 {
     if( d->packetSize <= 0 ) {
-        av_init_packet( &d->packet );
+        ::av_init_packet( &d->packet );
 
-        if( av_read_frame( d->formatContext, &d->packet ) < 0 ) {
+        if( ::av_read_frame( d->formatContext, &d->packet ) < 0 ) {
             return 0;
         }
-
         d->packetSize = d->packet.size;
         d->packetData = d->packet.data;
     }
@@ -293,27 +326,37 @@ int K3bFFMpegFile::fillOutputBuffer()
             return 0;
         }
 
-        d->outputBufferPos = d->outputBuffer;
+        d->outputBufferPos = d->alignedOutputBuffer;
+        d->outputBufferSize = AVCODEC_MAX_AUDIO_FRAME_SIZE;
 
-#if LIBAVCODEC_VERSION_MAJOR < 52
-        int len = avcodec_decode_audio(
+#ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO3
+        int len = ::avcodec_decode_audio3(
 #else
-        int len = avcodec_decode_audio2(
+#  ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO2
+        int len = ::avcodec_decode_audio2(
+#  else
+        int len = ::avcodec_decode_audio(
+#  endif
 #endif
-#ifdef FFMPEG_BUILD_PRE_4629
-            &d->formatContext->streams[0]->codec,
-#else
-            d->formatContext->streams[0]->codec,
-#endif
-            (short*)d->outputBuffer,
+
+            FFMPEG_CODEC(d->formatContext->streams[0]),
+            (short*)d->alignedOutputBuffer,
             &d->outputBufferSize,
+#ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO3
+            &d->packet );
+#else
             d->packetData, d->packetSize );
+#endif
+
+        if( d->packetSize <= 0 || len < 0 )
+            ::av_free_packet( &d->packet );
+        if( len < 0 ) {
+            kDebug() << "(K3bFFMpegFile) decoding failed for " << m_filename;
+            return -1;
+        }
 
         d->packetSize -= len;
         d->packetData += len;
-
-        if( d->packetSize <= 0  )
-            av_free_packet( &d->packet );
     }
 
     // if it is still empty try again
@@ -334,9 +377,9 @@ bool K3bFFMpegFile::seek( const K3b::Msf& msf )
 
     // FIXME: do we really need the start_time and why?
 #if LIBAVFORMAT_BUILD >= 4619
-    return ( av_seek_frame( d->formatContext, -1, timestamp + d->formatContext->start_time, 0 ) >= 0 );
+    return ( ::av_seek_frame( d->formatContext, -1, timestamp + d->formatContext->start_time, 0 ) >= 0 );
 #else
-    return ( av_seek_frame( d->formatContext, -1, timestamp + d->formatContext->start_time ) >= 0 );
+    return ( ::av_seek_frame( d->formatContext, -1, timestamp + d->formatContext->start_time ) >= 0 );
 #endif
 }
 
@@ -347,7 +390,7 @@ bool K3bFFMpegFile::seek( const K3b::Msf& msf )
 
 K3bFFMpegWrapper::K3bFFMpegWrapper()
 {
-    av_register_all();
+    ::av_register_all();
 }
 
 
